import './GameSelector.css';
import React, { useContext } from 'react';
import { GamesContext } from '../../../context/games';

import { Color, Player } from '../../../components/Checkers';
import Loading from '../../../components/Loading';
import Counter from '../../../components/Counter';

export function GameProposalSelector({ onSelect }) {
    const games = useContext(GamesContext);
    if (games.gamesList === null)
        return <Loading />

    const isSelected = (uuid) => uuid === games.proposal.selectedUUID;

    const onClick = (uuid) => {
        if (isSelected(uuid))
            onSelect(null); // deselect previously selected game
        else
            onSelect(uuid);
    }

    const yoursList = games.gamesList.filter(game => game.status === 'GAME_PROPOSAL_WAIT_FOR_YOU')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const opponentsList = games.gamesList.filter(game => game.status === 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    return (
        <div className='GameSelector'>
            {yoursList}
            {opponentsList.length > 0 && <Separator counter={opponentsList.length}>waiting for opponent</Separator>}
            {opponentsList}
        </div>
    )
}

export function ActiveGameSelector({ onSelect }) {
    const games = useContext(GamesContext);
    if (games.gamesList === null)
        return <Loading />

    const isSelected = (uuid) => uuid === games.active.selectedUUID;

    const onClick = (uuid) => {
        if (isSelected(uuid))
            onSelect(null); // deselect previously selected game
        else
            onSelect(uuid);
    }

    const yoursList = games.gamesList.filter(game => (game.status === 'GAME_BOARD_WAIT_FOR_YOU' || game.status === 'DRAW_REQUEST_WAIT_FOR_YOU'))
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const opponentsList = games.gamesList.filter(game => (game.status === 'GAME_BOARD_WAIT_FOR_OPPONENT' || game.status === 'DRAW_REQUEST_WAIT_FOR_OPPONENT'))
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    return (
        <div className='GameSelector'>
            {yoursList}
            {opponentsList.length > 0 && <Separator counter={opponentsList.length}>waiting for opponent</Separator>}
            {opponentsList}
        </div>
    )
}

export function GameArchiveSelector({ onSelect }) {
    const games = useContext(GamesContext);
    if (games.gamesList === null)
        return <Loading />

    const isSelected = (uuid) => uuid === games.archive.selectedUUID;

    const onClick = (uuid) => {
        if (isSelected(uuid))
            onSelect(null); // deselect previously selected game
        else
            onSelect(uuid);
    }

    const rejectedList = games.gamesList.filter(game => game.status === 'GAME_PROPOSAL_REJECTED')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const canceledList = games.gamesList.filter(game => game.status === 'GAME_PROPOSAL_CANCELED')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const victoryList = games.gamesList.filter(game => game.status === 'GAME_RESULT_YOU_WON')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const defeatList = games.gamesList.filter(game => game.status === 'GAME_RESULT_YOU_LOOSE')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const drawList = games.gamesList.filter(game => game.status === 'GAME_RESULT_DRAW')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    return (
        <div className='GameSelector'>
            {rejectedList.length > 0 && <Separator counter={rejectedList.length}>rejected proposals</Separator>}
            {rejectedList}
            {canceledList.length > 0 && <Separator counter={canceledList.length}>canceled proposals</Separator>}
            {canceledList}
            {victoryList.length > 0 && <Separator counter={victoryList.length}>victory</Separator>}
            {victoryList}
            {defeatList.length > 0 && <Separator counter={defeatList.length}>defeat</Separator>}
            {defeatList}
            {drawList.length > 0 && <Separator counter={drawList.length}>draw</Separator>}
            {drawList}
        </div>
    )
}

function Selectable({ game, selected, onClick }) {
    const myColor = game.myColor;
    const opponentColor = Color.opposite(myColor);
    const opponentName = game.opponentName;

    return (
        <div >
            <div className={'Selectable' + (selected ? ' selected' : '')}
                onClick={() => onClick(game.uuid)}>
                <div className='Title'>
                    <Player color={myColor} />
                    <i>vs</i>
                    <Player color={opponentColor} name={opponentName} />
                </div>
                <q>{game.message}</q>
            </div>
            <hr />
        </div>
    )
};

function Separator({ counter, children }) {
    return (
        <div className='Separator'>
            {children}
            <Counter number={counter} />
        </div>
    )
}